from unittest import TestCase
from unittest.mock import call, Mock

import click
from box import Box
from pycarlo.core import Client

from montecarlodata.monitors.monitor_service import MonitorService


class MonitorServiceTest(TestCase):
    MONITORS_TABLE = """\
╒════════════════╤═══════════╤═════════════╤══════════════════╤══════════════════════════════════╕
│ Monitor UUID   │ Type      │ Namespace   │ Description      │ Last Update Time                 │
╞════════════════╪═══════════╪═════════════╪══════════════════╪══════════════════════════════════╡
│ test_1         │ FRESHNESS │ main        │ some description │ 2000-01-01 00:00:00.000000+00:00 │
├────────────────┼───────────┼─────────────┼──────────────────┼──────────────────────────────────┤
│ test_2         │ FRESHNESS │ main        │ some description │ 2000-01-01 00:00:00.000000+00:00 │
╘════════════════╧═══════════╧═════════════╧══════════════════╧══════════════════════════════════╛"""
    LIMIT = 2

    def setUp(self):
        self._client = Mock(autospec=Client)
        self._print_func = Mock(autospec=click.echo)
        self._service = MonitorService(client=self._client, print_func=self._print_func)

    @staticmethod
    def _get_monitors_response(monitors_count):
        return Box(
            {
                "get_monitors": [
                    {
                        "uuid": f"test_{i}",
                        "monitor_type": "FRESHNESS",
                        "namespace": "main",
                        "description": "some description",
                        "last_update_time": "2000-01-01 00:00:00.000000+00:00",
                    }
                    for i in range(1, monitors_count + 1)
                ]
            }
        )

    def test_get_monitors(self):
        self._client.return_value = self._get_monitors_response(self.LIMIT)
        self._service.list_monitors(self.LIMIT)
        self._print_func.assert_called_once_with(self.MONITORS_TABLE)

    def test_get_monitors_with_more_available(self):
        self._client.return_value = self._get_monitors_response(self.LIMIT + 1)
        self._service.list_monitors(self.LIMIT)
        expected_calls = [
            call(self.MONITORS_TABLE),
            call(self._service.MORE_MONITOR_MESSAGE),
        ]
        self._print_func.assert_has_calls(expected_calls)



import json
from pycarlo.core import Client
from typing import Dict, List, Optional

from montecarlodata.config import Config
from montecarlodata.integrations.configure.fields import (
    MATCH_AND_CREATE_BI_WH_SRC_GQL_OPERATION,
    MATCH_AND_CREATE_BI_WH_SRC_GQL_RESPONSE_FIELD,
    MATCH_SUCCESSFUL_GQL_RESPONSE_FIELD,
    MATCHING_BI_WH_SRC_GQL_RESPONSE_FIELD,
)
from montecarlodata.queries.bi import MATCH_AND_CREATE_BI_WAREHOUSE_SOURCES
from montecarlodata.utils import GqlWrapper


class BiService:
    def __init__(
        self,
        config: Config,
        request_wrapper: Optional[GqlWrapper] = None,
        mc_client: Optional[Client] = None,
    ):
        self._request_wrapper = request_wrapper or GqlWrapper(config)
        self._mc_client = mc_client or Client()

    def refresh_bi_to_warehouse_connections(
        self, bi_container_id: str, warehouse_source_details: List[Dict]
    ) -> None:
        if warehouse_source_details:
            warehouse_source_details = [
                self._request_wrapper.convert_snakes_to_camels(wsd)
                for wsd in warehouse_source_details
            ]
            self._mc_client(
                query=MATCH_AND_CREATE_BI_WAREHOUSE_SOURCES,
                operation_name=MATCH_AND_CREATE_BI_WH_SRC_GQL_OPERATION,
                variables={
                    "biContainerId": bi_container_id,
                    "biWarehouseSources": warehouse_source_details,
                },
            )
        else:
            response = self._mc_client(
                query=MATCH_AND_CREATE_BI_WAREHOUSE_SOURCES,
                operation_name=MATCH_AND_CREATE_BI_WH_SRC_GQL_OPERATION,
                variables={
                    "biContainerId": bi_container_id,
                },
            )

            match_success = (
                response.get(MATCH_AND_CREATE_BI_WH_SRC_GQL_RESPONSE_FIELD, {})
                .get(MATCHING_BI_WH_SRC_GQL_RESPONSE_FIELD, {})
                .get(MATCH_SUCCESSFUL_GQL_RESPONSE_FIELD, False)
            )

            if not match_success:
                print(
                    "Matching failed. Please create the BI warehouse sources manually."
                )
                print(json.dumps(response, indent=2))
                return

        print("Operation completed successfully.")



from unittest import TestCase
from unittest.mock import Mock, patch

from montecarlodata.common.user import UserService
from montecarlodata.integrations.onboarding.data_lake.presto import (
    PrestoOnboardingService,
)
from montecarlodata.queries.onboarding import (
    TEST_PRESTO_CRED_MUTATION,
    TEST_S3_CRED_MUTATION,
)
from montecarlodata.utils import GqlWrapper, AwsClientWrapper
from tests.test_base_onboarding import _SAMPLE_BASE_OPTIONS
from tests.test_common_user import _SAMPLE_CONFIG


class PrestoOnBoardingTest(TestCase):
    def setUp(self) -> None:
        self._user_service_mock = Mock(autospec=UserService)
        self._request_wrapper_mock = Mock(autospec=GqlWrapper)
        self._aws_wrapper_mock = Mock(autospec=AwsClientWrapper)

        self._service = PrestoOnboardingService(
            _SAMPLE_CONFIG,
            request_wrapper=self._request_wrapper_mock,
            aws_wrapper=self._aws_wrapper_mock,
            user_service=self._user_service_mock,
        )

    @patch.object(PrestoOnboardingService, "onboard")
    def test_presto_sql_flow(self, onboard_mock):
        s3_key = "testing/test"
        expected_options = {
            "ssl_options": {
                "mechanism": "dc-s3",
                "cert": "testing/test",
                "skip_verification": False,
            },
            **_SAMPLE_BASE_OPTIONS,
        }

        self._service.onboard_presto_sql(
            **{
                "cert_s3": s3_key,
                "skip_cert_verification": False,
                **_SAMPLE_BASE_OPTIONS,
            }
        )
        onboard_mock.assert_called_once_with(
            validation_query=TEST_PRESTO_CRED_MUTATION,
            validation_response="testPrestoCredentials",
            connection_type="presto",
            **expected_options
        )

    @patch.object(PrestoOnboardingService, "onboard")
    def test_presto_s3_flow(self, onboard_mock):
        expected_options = {**_SAMPLE_BASE_OPTIONS, **{"connectionType": "presto-s3"}}

        self._service.onboard_presto_s3(**_SAMPLE_BASE_OPTIONS)
        onboard_mock.assert_called_once_with(
            validation_query=TEST_S3_CRED_MUTATION,
            validation_response="testS3Credentials",
            connection_type="presto-s3",
            job_types=["query_logs"],
            **expected_options
        )



import json
from typing import Optional, Dict

from montecarlodata.iac.schemas import (
    ConfigTemplateUpdateResponse,
    ConfigTemplateDeleteResponse,
)
from montecarlodata.queries.iac import (
    CREATE_OR_UPDATE_MONTE_CARLO_CONFIG_TEMPLATE,
    DELETE_MONTE_CARLO_CONFIG_TEMPLATE,
)
from montecarlodata.utils import GqlWrapper


class MonteCarloConfigTemplateClient:
    def __init__(self, gql_wrapper: GqlWrapper):
        self._gql_wrapper = gql_wrapper

    def apply_config_template(
        self,
        namespace: str,
        config_template_as_dict: Dict,
        resource: Optional[str] = None,
        dry_run: bool = False,
        misconfigured_as_warning: bool = False,
    ) -> ConfigTemplateUpdateResponse:
        response = self._gql_wrapper.make_request_v2(
            query=CREATE_OR_UPDATE_MONTE_CARLO_CONFIG_TEMPLATE,
            operation="createOrUpdateMonteCarloConfigTemplate",
            variables=dict(
                namespace=namespace,
                configTemplateJson=json.dumps(config_template_as_dict),
                dryRun=dry_run,
                misconfiguredAsWarning=misconfigured_as_warning,
                resource=resource,
            ),
        )

        return ConfigTemplateUpdateResponse.from_dict(response.data["response"])

    def delete_config_template(
        self, namespace: str, dry_run: bool = False
    ) -> ConfigTemplateDeleteResponse:
        response = self._gql_wrapper.make_request_v2(
            query=DELETE_MONTE_CARLO_CONFIG_TEMPLATE,
            operation="deleteMonteCarloConfigTemplate",
            variables=dict(
                namespace=namespace,
                dryRun=dry_run,
            ),
        )

        return ConfigTemplateDeleteResponse.from_dict(response.data["response"])



import click

from montecarlodata.common import create_mc_client
from montecarlodata.management.service import ManagementService


@click.group(help="Manage account settings.")
def management():
    """
    Group for any management related subcommands
    """
    pass


@management.command(help="Get PII filtering preferences.")
@click.pass_obj
def get_pii_preferences(ctx):
    ManagementService(mc_client=create_mc_client(ctx)).get_pii_preferences()


@management.command(help="Configure PII filtering preferences for the account.")
@click.option(
    "--enable/--disable",
    "enabled",
    required=False,
    type=click.BOOL,
    default=None,
    help="Whether PII filtering should be active for the account.",
)
@click.option(
    "--fail-mode",
    required=False,
    type=click.Choice(["CLOSE", "OPEN"], case_sensitive=False),
    help="Whether PII filter failures will allow (OPEN) or prevent (CLOSE) data flow for this account.",
)
@click.pass_obj
def configure_pii_filtering(ctx, **kwargs):
    ManagementService(mc_client=create_mc_client(ctx)).set_pii_filtering(**kwargs)



# Query responses
EXPECTED_ADD_CONNECTION_RESPONSE_FIELD = "addConnection"
EXPECTED_GENERIC_DB_GQL_RESPONSE_FIELD = "testDatabaseCredentials"
EXPECTED_HIVE_S3_GQL_RESPONSE_FIELD = "testS3Credentials"
EXPECTED_HIVE_SQL_GQL_RESPONSE_FIELD = "testHiveCredentials"
EXPECTED_PRESTO_SQL_GQL_RESPONSE_FIELD = "testPrestoCredentials"
EXPECTED_PRESTO_S3_GQL_RESPONSE_FIELD = "testS3Credentials"
EXPECTED_GLUE_GQL_RESPONSE_FIELD = "testGlueCredentials"
EXPECTED_ATHENA_GQL_RESPONSE_FIELD = "testAthenaCredentials"
EXPECTED_SPARK_GQL_RESPONSE_FIELD = "testSparkCredentials"
EXPECTED_DATABRICKS_GQL_RESPONSE_FIELD = "testDatabricksCredentials"
EXPECTED_SNOWFLAKE_GQL_RESPONSE_FIELD = "testSnowflakeCredentials"
EXPECTED_BQ_GQL_RESPONSE_FIELD = "testBqCredentials"
EXPECTED_SELF_HOSTED_GQL_RESPONSE_FIELD = "testSelfHostedCredentials"
EXPECTED_CONFIGURE_AIRFLOW_LOG_EVENTS_GQL_RESPONSE_FIELD = "configureAirflowLogEvents"
EXPECTED_CONFIGURE_METADATA_EVENTS_GQL_RESPONSE_FIELD = "configureMetadataEvents"
EXPECTED_CONFIGURE_QUERY_LOG_EVENTS_GQL_RESPONSE_FIELD = "configureQueryLogEvents"
EXPECTED_DISABLE_AIRFLOW_LOG_EVENTS_GQL_RESPONSE_FIELD = "disableAirflowLogEvents"
EXPECTED_DISABLE_METADATA_EVENTS_GQL_RESPONSE_FIELD = "disableMetadataEvents"
EXPECTED_DISABLE_QUERY_LOG_EVENTS_GQL_RESPONSE_FIELD = "disableQueryLogEvents"
EXPECTED_UPDATE_CREDENTIALS_RESPONSE_FIELD = "updateCredentials"
EXPECTED_REMOVE_CONNECTION_RESPONSE_FIELD = "removeConnection"
EXPECTED_TEST_EXISTING_RESPONSE_FIELD = "testExistingConnection"
EXPECTED_TEST_TABLEAU_RESPONSE_FIELD = "testTableauCredentials"
EXPECTED_LOOKER_METADATA_RESPONSE_FIELD = "testLookerCredentials"
EXPECTED_LOOKER_GIT_SSH_RESPONSE_FILED = "testLookerGitSshCredentials"
EXPECTED_LOOKER_GIT_CLONE_RESPONSE_FIELD = "testLookerGitCloneCredentials"
EXPECTED_POWER_BI_RESPONSE_FIELD = "testPowerBiCredentials"
EXPECTED_DBT_CLOUD_RESPONSE_FIELD = "testDbtCloudCredentials"
EXPECTED_ADD_BI_RESPONSE_FIELD = "addBiConnection"
EXPECTED_FIVETRAN_RESPONSE_FIELD = "testFivetranCredentials"

# Available connections types
AIRFLOW_LOGS_CONNECTION_TYPE = "s3-airflow-log-events"
ATHENA_CONNECTION_TYPE = "athena"
BQ_CONNECTION_TYPE = "bigquery"
DATABRICKS_DELTA_CONNECTION_TYPE = "databricks-delta"
DATABRICKS_METASTORE_CONNECTION_TYPE = "databricks-metastore"
DBT_CLOUD_CONNECTION_TYPE = "dbt-cloud-v2"
DBT_CORE_CONNECTION_TYPE = "dbt-core"
FIVETRAN_CONNECTION_TYPE = "fivetran"
GLUE_CONNECTION_TYPE = "glue"
HIVE_MYSQL_CONNECTION_TYPE = "hive-mysql"
HIVE_S3_CONNECTION_TYPE = "hive-s3"
HIVE_SQL_CONNECTION_TYPE = "hive"
LOOKER_GIT_CLONE_CONNECTION_TYPE = "looker-git-clone"
LOOKER_GIT_SSH_CONNECTION_TYPE = "looker-git-ssh"
LOOKER_MD_CONNECTION_TYPE = "looker"
POWER_BI_CONNECTION_TYPE = "power-bi"
PRESTO_S3_CONNECTION_TYPE = "presto-s3"
PRESTO_SQL_CONNECTION_TYPE = "presto"
REDSHIFT_CONNECTION_TYPE = "redshift"
SNOWFLAKE_CONNECTION_TYPE = "snowflake"
SPARK_CONNECTION_TYPE = "spark"
TABLEAU_CONNECTION_TYPE = "tableau"

METASTORE_CONNECTION_TYPES = (
    HIVE_MYSQL_CONNECTION_TYPE,
    GLUE_CONNECTION_TYPE,
    DATABRICKS_METASTORE_CONNECTION_TYPE,
)

# Every warehouse must have one and only one of these.
MAIN_CONNECTION_TYPES = frozenset(
    [
        BQ_CONNECTION_TYPE,
        REDSHIFT_CONNECTION_TYPE,
        SNOWFLAKE_CONNECTION_TYPE,
        *METASTORE_CONNECTION_TYPES,
    ]
)

# Available warehouse types
DATA_LAKE_WAREHOUSE_TYPE = "data-lake"
REDSHIFT_WAREHOUSE_TYPE = REDSHIFT_CONNECTION_TYPE
SNOWFLAKE_WAREHOUSE_TYPE = SNOWFLAKE_CONNECTION_TYPE
BQ_WAREHOUSE_TYPE = BQ_CONNECTION_TYPE

CONNECTION_TO_WAREHOUSE_TYPE_MAP = {
    BQ_CONNECTION_TYPE: BQ_WAREHOUSE_TYPE,
    REDSHIFT_CONNECTION_TYPE: REDSHIFT_WAREHOUSE_TYPE,
    SNOWFLAKE_CONNECTION_TYPE: SNOWFLAKE_WAREHOUSE_TYPE,
}

# Available BI types
LOOKER_BI_TYPE = "looker"
TABLEAU_BI_TYPE = "tableau"
POWER_BI_BI_TYPE = "power-bi"

# Available credential self-hosting mechanisms
SECRETS_MANAGER_CREDENTIAL_MECHANISM = "secretsmanager"
SELF_HOSTING_MECHANISMS = [SECRETS_MANAGER_CREDENTIAL_MECHANISM]

# S3 event types
AIRFLOW_LOGS_EVENT_TYPE = "s3_airflow_log_events"
S3_METADATA_EVENT_TYPE = "s3_metadata_events"
S3_QL_EVENT_TYPE = "s3_ql_events"

# Job types
QL_JOB_TYPE = ["query_logs"]

# Job limits
PRESTO_CATALOG_KEY = "catalog_name"
HIVE_GET_PARTS_KEY = "get_partition_locations"
HIVE_MAX_PARTS_KEY = "max_partition_locations"
HIVE_MAX_PARTS_DEFAULT_VALUE = 50

# Certificate details
S3_CERT_MECHANISM = "dc-s3"
PRESTO_CERT_PREFIX = "certificates/presto/"
AWS_RDS_CA_CERT = "https://s3.amazonaws.com/rds-downloads/rds-combined-ca-bundle.pem"

# Connections to friendly name (i.e. human presentable) map
GQL_TO_FRIENDLY_CONNECTION_MAP = {
    AIRFLOW_LOGS_CONNECTION_TYPE: "Airflow Logs",
    ATHENA_CONNECTION_TYPE: "Athena",
    BQ_CONNECTION_TYPE: "BigQuery",
    DATABRICKS_DELTA_CONNECTION_TYPE: "Databricks (Delta lake)",
    DATABRICKS_METASTORE_CONNECTION_TYPE: "Databricks (metastore)",
    DBT_CLOUD_CONNECTION_TYPE: "dbt Cloud",
    DBT_CORE_CONNECTION_TYPE: "dbt Core",
    FIVETRAN_CONNECTION_TYPE: "Fivetran",
    GLUE_CONNECTION_TYPE: GLUE_CONNECTION_TYPE.capitalize(),
    HIVE_MYSQL_CONNECTION_TYPE: "Hive (metastore)",
    HIVE_S3_CONNECTION_TYPE: "Hive (EMR logs)",
    HIVE_SQL_CONNECTION_TYPE: "Hive (SQL)",
    LOOKER_GIT_CLONE_CONNECTION_TYPE: "Looker ML (git)",
    LOOKER_GIT_SSH_CONNECTION_TYPE: "Looker ML (git)",
    LOOKER_MD_CONNECTION_TYPE: "Looker (metadata)",
    POWER_BI_CONNECTION_TYPE: "Power BI",
    PRESTO_S3_CONNECTION_TYPE: "Presto (logs)",
    PRESTO_SQL_CONNECTION_TYPE: PRESTO_SQL_CONNECTION_TYPE.capitalize(),
    REDSHIFT_CONNECTION_TYPE: REDSHIFT_CONNECTION_TYPE.capitalize(),
    SNOWFLAKE_CONNECTION_TYPE: SNOWFLAKE_CONNECTION_TYPE.capitalize(),
    SPARK_CONNECTION_TYPE: "Spark (SQL)",
    TABLEAU_CONNECTION_TYPE: "Tableau",
}

# Verbiage
CONNECTION_TEST_SUCCESS_VERBIAGE = "Connection test was successful!"
CONNECTION_TEST_FAILED_VERBIAGE = "Connection test failed!"
ADD_CONNECTION_SUCCESS_VERBIAGE = "Success! Added connection for "
ADD_CONNECTION_FAILED_VERBIAGE = "Failed to add connection!"
VALIDATIONS_FAILED_VERBIAGE = (
    "Some validations failed. Would you like to create the connection anyway?"
)
OPERATION_ERROR_VERBIAGE = (
    "Operation failed - This might not be a valid connection for your account. "
    "Please contact Monte Carlo."
)
CONFIRM_CONNECTION_VERBIAGE = "Please confirm you want to add this connection"
SKIP_ADD_CONNECTION_VERBIAGE = "Skipping adding the connection."



from montecarlodata.common.data import AWSArn
from montecarlodata.errors import complain_and_abort, manage_errors
from montecarlodata.integrations.onboarding.base import BaseOnboardingService
from montecarlodata.integrations.onboarding.fields import (
    CONNECTION_TO_WAREHOUSE_TYPE_MAP,
    EXPECTED_ADD_BI_RESPONSE_FIELD,
    EXPECTED_SELF_HOSTED_GQL_RESPONSE_FIELD,
    LOOKER_GIT_CLONE_CONNECTION_TYPE,
    LOOKER_MD_CONNECTION_TYPE,
)
from montecarlodata.queries.onboarding import (
    ADD_BI_CONNECTION_MUTATION,
    TEST_SELF_HOSTED_CRED_MUTATION,
)


class SelfHostedCredentialOnboardingService(BaseOnboardingService):
    BI_CONNECTION_TYPES = {LOOKER_GIT_CLONE_CONNECTION_TYPE, LOOKER_MD_CONNECTION_TYPE}

    @manage_errors
    def onboard_connection(
        self, connection_type: str, self_hosting_key: str, **kwargs
    ) -> None:
        """
        Onboard a connection with self-hosted credentials by validating and adding a connection
        """
        kwargs["connectionType"] = connection_type

        try:
            region = AWSArn(self_hosting_key).region
        except IndexError:
            raise complain_and_abort("Credential key is not a valid ARN")

        if connection_type in CONNECTION_TO_WAREHOUSE_TYPE_MAP:
            kwargs["warehouseType"] = CONNECTION_TO_WAREHOUSE_TYPE_MAP[connection_type]
        elif connection_type in self.BI_CONNECTION_TYPES:
            kwargs["connection_query"] = ADD_BI_CONNECTION_MUTATION
            kwargs["connection_response"] = EXPECTED_ADD_BI_RESPONSE_FIELD

        self.onboard(
            validation_query=TEST_SELF_HOSTED_CRED_MUTATION,
            validation_response=EXPECTED_SELF_HOSTED_GQL_RESPONSE_FIELD,
            connection_type=connection_type,
            self_hosting_key=self_hosting_key,
            region=region,
            **kwargs
        )



from typing import Optional

import click
from pycarlo.core import Client, Mutation, Query
from tabulate import tabulate

from montecarlodata.common.common import ConditionalDictionary
from montecarlodata.errors import manage_errors


class ManagementService:
    _LIST_PII_PREFERENCES_HEADERS = ["Status", "Fail Mode"]

    def __init__(self, mc_client: Optional[Client] = None):
        self._abort_on_error = True
        self._mc_client = mc_client or Client()

    @manage_errors
    def get_pii_preferences(
        self,
        headers: Optional[str] = "firstrow",
        table_format: Optional[str] = "fancy_grid",
    ) -> None:
        table = [self._LIST_PII_PREFERENCES_HEADERS]

        query = Query()
        query.get_pii_filtering_preferences()
        preferences = self._mc_client(query).get_pii_filtering_preferences
        table.append(
            [
                "Enabled" if preferences.enabled else "Disabled",
                preferences.fail_mode,
            ]
        )
        click.echo(tabulate(table, headers=headers, tablefmt=table_format))

    @manage_errors
    def set_pii_filtering(
        self,
        enabled: Optional[bool] = None,
        fail_mode: Optional[str] = None,
    ) -> None:
        variables = ConditionalDictionary(lambda x: x is not None)
        variables.update({"enabled": enabled, "fail_mode": fail_mode})

        mutation = Mutation()
        mutation.update_pii_filtering_preferences(**variables)
        result = self._mc_client(mutation).update_pii_filtering_preferences
        if result.success:
            click.echo("PII filtering preferences have been updated!")



from typing import Dict, Optional

import click

from montecarlodata.common.common import read_as_json_string
from montecarlodata.config import Config
from montecarlodata.errors import complain_and_abort, manage_errors
from montecarlodata.integrations.onboarding.base import BaseOnboardingService
from montecarlodata.integrations.onboarding.fields import (
    EXPECTED_CONFIGURE_AIRFLOW_LOG_EVENTS_GQL_RESPONSE_FIELD,
    EXPECTED_CONFIGURE_METADATA_EVENTS_GQL_RESPONSE_FIELD,
    EXPECTED_CONFIGURE_QUERY_LOG_EVENTS_GQL_RESPONSE_FIELD,
    EXPECTED_DISABLE_AIRFLOW_LOG_EVENTS_GQL_RESPONSE_FIELD,
    EXPECTED_DISABLE_METADATA_EVENTS_GQL_RESPONSE_FIELD,
    EXPECTED_DISABLE_QUERY_LOG_EVENTS_GQL_RESPONSE_FIELD,
)
from montecarlodata.queries.onboarding import (
    CONFIGURE_AIRFLOW_LOG_EVENTS_MUTATION,
    CONFIGURE_METADATA_EVENTS_MUTATION,
    CONFIGURE_QUERY_LOG_EVENTS_MUTATION,
    DISABLE_AIRFLOW_LOG_EVENTS_MUTATION,
    DISABLE_METADATA_EVENTS_MUTATION,
    DISABLE_QUERY_LOG_EVENTS_MUTATION,
)


class EventsOnboardingService(BaseOnboardingService):
    def __init__(self, config: Config, **kwargs):
        super().__init__(config, **kwargs)

    @manage_errors
    def configure_airflow_events(self, **kwargs):
        self._configure_events(
            query=CONFIGURE_AIRFLOW_LOG_EVENTS_MUTATION,
            operation=EXPECTED_CONFIGURE_AIRFLOW_LOG_EVENTS_GQL_RESPONSE_FIELD,
            variables=kwargs,
        )

    @manage_errors
    def configure_metadata_events(self, **kwargs):
        self._configure_events(
            query=CONFIGURE_METADATA_EVENTS_MUTATION,
            operation=EXPECTED_CONFIGURE_METADATA_EVENTS_GQL_RESPONSE_FIELD,
            variables=kwargs,
        )

    @manage_errors
    def configure_query_log_events(self, **kwargs):
        mapping_file = kwargs.pop("mapping_file", None)
        if mapping_file:
            kwargs["mapping"] = read_as_json_string(mapping_file)

        self._configure_events(
            query=CONFIGURE_QUERY_LOG_EVENTS_MUTATION,
            operation=EXPECTED_CONFIGURE_QUERY_LOG_EVENTS_GQL_RESPONSE_FIELD,
            variables=kwargs,
        )

    @manage_errors
    def disable_airflow_log_events(self, **kwargs):
        self._disable_events(
            query=DISABLE_AIRFLOW_LOG_EVENTS_MUTATION,
            operation=EXPECTED_DISABLE_AIRFLOW_LOG_EVENTS_GQL_RESPONSE_FIELD,
            variables=kwargs,
        )

    @manage_errors
    def disable_metadata_events(self, **kwargs):
        self._disable_events(
            query=DISABLE_METADATA_EVENTS_MUTATION,
            operation=EXPECTED_DISABLE_METADATA_EVENTS_GQL_RESPONSE_FIELD,
            variables=kwargs,
        )

    @manage_errors
    def disable_query_log_events(self, **kwargs):
        self._disable_events(
            query=DISABLE_QUERY_LOG_EVENTS_MUTATION,
            operation=EXPECTED_DISABLE_QUERY_LOG_EVENTS_GQL_RESPONSE_FIELD,
            variables=kwargs,
        )

    def _configure_events(
        self, query: str, operation: str, variables: Optional[Dict] = None
    ):

        self._do_configure_events(
            action="configure", query=query, operation=operation, variables=variables
        )

    def _disable_events(
        self, query: str, operation: str, variables: Optional[Dict] = None
    ):

        self._do_configure_events(
            action="disable", query=query, operation=operation, variables=variables
        )

    def _do_configure_events(
        self, action: str, query: str, operation: str, variables: Optional[Dict] = None
    ):

        response = self._request_wrapper.make_request_v2(query, operation, variables)
        if response.data.success:
            return click.echo(f"Successfully {action}d events!")

        complain_and_abort(f"Failed to {action} events!")



import os
import urllib.request
from pathlib import Path
from typing import Optional
from urllib.parse import urlparse

import click
import requests
from tabulate import tabulate

from montecarlodata.config import Config
from montecarlodata.errors import manage_errors, complain_and_abort
from montecarlodata.fs_utils import mkdirs
from montecarlodata.insights.fields import (
    EXPECTED_GET_INSIGHTS_FIELD,
    FILE_SCHEME,
    S3_SCHEME,
    EXPECTED_GET_REPORT_FIELD,
    INSIGHTS_DEFAULT_EXTENSION,
    SCHEME_DELIM,
    LIST_INSIGHTS_HEADERS,
)
from montecarlodata.queries.insights import GET_INSIGHTS, GET_INSIGHT_REPORT
from montecarlodata.utils import GqlWrapper, AwsClientWrapper


class InsightsService:
    def __init__(self, config: Config, request_wrapper: Optional[GqlWrapper] = None):
        self._abort_on_error = True
        self._profile = config.aws_profile

        self._request_wrapper = request_wrapper or GqlWrapper(config)

        self.scheme_handler = {
            FILE_SCHEME: self._save_insight_to_disk,
            S3_SCHEME: self._save_insight_to_s3,
        }

    @manage_errors
    def echo_insights(
        self,
        headers: Optional[str] = "firstrow",
        table_format: Optional[str] = "fancy_grid",
    ) -> None:
        """
        Echo insights as a pretty table
        """
        table = [LIST_INSIGHTS_HEADERS]
        for insight in self._request_wrapper.make_request_v2(
            query=GET_INSIGHTS, operation=EXPECTED_GET_INSIGHTS_FIELD
        ).data:
            table.append(
                [
                    insight.title,
                    insight.description
                    if insight.description.endswith(".")
                    else f"{insight.description}.",
                    insight.available,
                ]
            )
        click.echo(tabulate(table, headers=headers, tablefmt=table_format))

    @manage_errors
    def get_insight(
        self,
        insight: str,
        destination: str,
        aws_profile: Optional[str] = None,
        dry: Optional[bool] = False,
    ) -> None:
        """
        Get insight, if available, and either persist to S3 or local file system
        """
        parsed_destination = urlparse(destination)
        scheme = parsed_destination.scheme
        netloc_with_path = parsed_destination.geturl().replace(
            f"{scheme}{SCHEME_DELIM}", "", 1
        )

        try:
            handler = self.scheme_handler[scheme]
        except KeyError:
            complain_and_abort("Scheme either missing or not supported.")
        else:
            insight_url = self._get_insight_url(insight=insight)
            if dry:
                click.echo(insight_url)
                return
            click.echo(f"Saving insight to '{destination}'.")
            handler(
                insight_url=insight_url,
                destination=netloc_with_path,
                aws_profile=aws_profile,
            )
            click.echo("Complete. Have a nice day!")

    def _get_insight_url(self, insight: str) -> str:
        """
        Get insight URL from the monolith
        """
        url = self._request_wrapper.make_request_v2(
            query=GET_INSIGHT_REPORT,
            operation=EXPECTED_GET_REPORT_FIELD,
            variables=dict(
                insight_name=insight,
                report_name=str(Path(insight).with_suffix(INSIGHTS_DEFAULT_EXTENSION)),
            ),
        ).data.url
        if not url:
            complain_and_abort(
                "Insight not found. This insight might not be available for your account."
            )
        return url

    def _save_insight_to_disk(
        self, insight_url: str, destination: str, **kwargs
    ) -> None:
        """
        Save insight to the local filesystem
        """
        if not destination:
            complain_and_abort(
                f"Invalid path. Expected format: '{FILE_SCHEME}{SCHEME_DELIM}folder/file.csv'"
            )
        mkdirs(os.path.dirname(destination))
        urllib.request.urlretrieve(insight_url, destination)

    def _save_insight_to_s3(
        self, insight_url: str, destination: str, aws_profile: Optional[str] = None
    ) -> None:
        """
        Save insight to S3
        """
        try:
            bucket, key = destination.split("/", maxsplit=1)
            if not key:
                raise ValueError
        except (ValueError, AttributeError):
            complain_and_abort(
                f"Invalid path. Expected format: '{S3_SCHEME}{SCHEME_DELIM}bucket/key.csv'"
            )
        else:
            AwsClientWrapper(
                profile_name=aws_profile or self._profile
            ).upload_stream_to_s3(
                data=requests.get(insight_url, stream=True).raw, bucket=bucket, key=key
            )



import json
from typing import Optional, Dict
from uuid import UUID

import click

from montecarlodata.common.data import MonolithResponse
from montecarlodata.config import Config
from montecarlodata.errors import manage_errors
from montecarlodata.integrations.onboarding.base import BaseOnboardingService
from montecarlodata.integrations.onboarding.fields import (
    EXPECTED_UPDATE_CREDENTIALS_RESPONSE_FIELD,
    EXPECTED_REMOVE_CONNECTION_RESPONSE_FIELD,
    OPERATION_ERROR_VERBIAGE,
    EXPECTED_TEST_EXISTING_RESPONSE_FIELD,
)
from montecarlodata.queries.onboarding import (
    UPDATE_CREDENTIALS_MUTATION,
    REMOVE_CONNECTION_MUTATION,
    TEST_EXISTING_CONNECTION_QUERY,
    ConnectionOperationsQueries,
)


class ConnectionOperationsService(BaseOnboardingService):
    REMOVE_PROMPT = (
        "Are you sure? This action deletes any associated jobs, monitors, etc."
    )

    def __init__(self, config: Config, **kwargs):
        super().__init__(config, **kwargs)

    @manage_errors
    def update_credentials(
        self,
        connection_id: UUID,
        changes: Dict,
        should_validate: Optional[bool] = True,
        should_replace: Optional[bool] = False,
    ) -> None:
        """
        Update credentials for a connection
        """
        self.echo_operation_status(
            self._request_wrapper.make_request_v2(
                query=UPDATE_CREDENTIALS_MUTATION,
                operation=EXPECTED_UPDATE_CREDENTIALS_RESPONSE_FIELD,
                variables=dict(
                    connection_id=str(connection_id),
                    changes=json.dumps(changes),
                    should_validate=should_validate,
                    should_replace=should_replace,
                ),
            ),
            operation=f"Updated '{connection_id}'.",
        )

    @manage_errors
    def remove_connection(
        self, connection_id: UUID, no_prompt: Optional[bool] = False
    ) -> None:
        """
        Remove connection by ID. Deletes any associated jobs
        """
        if no_prompt or click.confirm(self.REMOVE_PROMPT, abort=True):
            self.echo_operation_status(
                self._request_wrapper.make_request_v2(
                    query=REMOVE_CONNECTION_MUTATION,
                    operation=EXPECTED_REMOVE_CONNECTION_RESPONSE_FIELD,
                    variables=dict(connection_id=str(connection_id)),
                ),
                operation=f"Removed '{connection_id}'.",
            )

    @manage_errors
    def echo_test_existing(self, connection_id: UUID) -> None:
        """
        Tests an existing connection and echos results in pretty JSON
        """
        response = self._request_wrapper.make_request_v2(
            query=TEST_EXISTING_CONNECTION_QUERY,
            operation=EXPECTED_TEST_EXISTING_RESPONSE_FIELD,
            variables=dict(connection_id=connection_id),
        )
        click.echo(json.dumps(response.data, indent=4, ensure_ascii=False))

    @staticmethod
    def echo_operation_status(response: MonolithResponse, operation: str) -> None:
        """
        Echos operation status. Expects response contains a 'success' field as per API signature
        """
        if response.data.get("success"):
            click.echo(f"Success! {operation}")
        else:
            # Any errors thrown by the API are captured and handled in the request
            # lib/caller deco (e.g. validation errors). This should only reachable for invalid inputs for the account
            click.echo(OPERATION_ERROR_VERBIAGE)

    @manage_errors
    def set_warehouse_name(self, current_name: str, new_name: str) -> None:
        """
        Sets the name of a warehouse
        """

        dw_id = self._get_warehouse_id_from_name(name=current_name)
        response = self._request_wrapper.make_request_v2(
            query=ConnectionOperationsQueries.set_warehouse_name.query,
            operation=ConnectionOperationsQueries.set_warehouse_name.operation,
            variables=dict(dw_id=dw_id, name=new_name),
        )

        if response.data.warehouse.name == new_name:
            click.echo(f"Success! Set Warehouse {current_name} name to {new_name}")
        else:
            click.echo(OPERATION_ERROR_VERBIAGE)

    @manage_errors
    def set_bi_connection_name(self, bi_connection_id: str, new_name: str) -> None:
        """
        Sets the name of a BI Connection
        """
        bi_container_id = self._get_bi_container_id_from_connection_id(
            bi_connection_id=bi_connection_id
        )
        response = self._request_wrapper.make_request_v2(
            query=ConnectionOperationsQueries.set_bi_connection_name.query,
            operation=ConnectionOperationsQueries.set_bi_connection_name.operation,
            variables=dict(resource_id=bi_container_id, name=new_name),
        )

        if response.data.biContainer.name == new_name:
            click.echo(f"Success! Set BI Connection name to {new_name}")
        else:
            click.echo(OPERATION_ERROR_VERBIAGE)



import click
from typing import Optional, Union
from uuid import UUID

from pycarlo.core.client import Client
from pycarlo.features.dbt import DbtImporter
from pycarlo.features.exceptions import MultipleResourcesFoundException
from pycarlo.features.pii import PiiService

from montecarlodata.common.user import UserService
from montecarlodata.config import Config
from montecarlodata.errors import complain_and_abort, manage_errors


class DbtImportService:
    def __init__(
        self,
        config: Config,
        mc_client: Optional[Client] = None,
        user_service: Optional[UserService] = None,
        pii_service: Optional[PiiService] = None,
    ):

        self._mc_client = mc_client or Client()
        self._user_service = user_service or UserService(config)
        self._pii_service = pii_service or PiiService(self._mc_client)

    @manage_errors
    def import_run(
        self,
        project_name: str,
        job_name: str,
        manifest_path: str,
        run_results_path: str,
        logs_path: Optional[str],
        connection_id: Optional[UUID],
    ):

        # find resource id associated with given connection id
        resource_id = self._find_resource_id(connection_id) if connection_id else None

        try:
            # execute import
            DbtImporter(
                mc_client=self._mc_client,
                print_func=click.echo,
                pii_service=self._pii_service,
            ).import_run(
                project_name=project_name,
                job_name=job_name,
                manifest_path=manifest_path,
                run_results_path=run_results_path,
                logs_path=logs_path,
                resource_id=resource_id,
            )
        except MultipleResourcesFoundException:
            # if multiple resources exist, ask user to specify a connection id
            complain_and_abort(
                "Multiple resources found, please specify a connection id."
            )

    def _find_resource_id(self, connection_id: UUID) -> Optional[str]:
        warehouse = self._user_service.get_warehouse_for_connection(connection_id)
        if warehouse is None:
            complain_and_abort(f"Could not find a connection with id: {connection_id}")
        return warehouse.uuid



import click
import click_config_file

from montecarlodata import settings
from montecarlodata.iac.mc_config_service import MonteCarloConfigService
from montecarlodata.monitors.monitor_service import MonitorService


@click.group(help="Manage monitors.")
def monitors():
    """
    Group for any monitor related subcommands
    """
    pass


@monitors.command(help="Compile monitor configuration.")
@click.option(
    "--project-dir",
    required=False,
    help="Base directory of MC project (where montecarlo.yml is located). By default, this is set to the current working directory",
)
@click.pass_obj
def compile(ctx, project_dir):
    MonteCarloConfigService(config=ctx["config"], project_dir=project_dir).compile()


@monitors.command(help="Compile and apply monitor configuration.")
@click.option(
    "--project-dir",
    required=False,
    help="Base directory of MC project (where montecarlo.yml is located). By default, this is set to the current working directory",
)
@click.option("--namespace", required=True, help="Namespace of monitors configuration.")
@click.option(
    "--dry-run",
    required=False,
    default=False,
    show_default=True,
    is_flag=True,
    help="Dry run (just shows planned changes but doesn't apply them.",
)
@click.pass_obj
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def apply(ctx, project_dir, namespace, dry_run):
    MonteCarloConfigService(config=ctx["config"], project_dir=project_dir).apply(
        namespace, dry_run=dry_run
    )


@monitors.command(help="Delete monitor configuration.")
@click.option(
    "--project-dir",
    required=False,
    help="Base directory of MC project (where montecarlo.yml is located). By default, this is set to the current working directory",
)
@click.option("--namespace", required=True, help="Namespace of monitors configuration.")
@click.option(
    "--dry-run",
    required=False,
    default=False,
    show_default=True,
    is_flag=True,
    help="Dry run (just shows planned changes but doesn't apply them.",
)
@click.pass_obj
@click_config_file.configuration_option(settings.OPTION_FILE_FLAG)
def delete(ctx, project_dir, namespace, dry_run):
    MonteCarloConfigService(config=ctx["config"], project_dir=project_dir).delete(
        namespace, dry_run=dry_run
    )


@monitors.command(name="list", help="List monitors ordered by update recency.")
@click.option(
    "--limit",
    required=False,
    default=100,
    type=click.INT,
    show_default=True,
    help="Max number of monitors to list.",
)
@click.option(
    "--monitor-type",
    required=False,
    type=click.Choice(MonitorService.MONITOR_TYPES, case_sensitive=False),
    help="List monitors with monitor_type",
)
@click.option(
    "--namespace", required=False, help="List only monitors in this namespace"
)
@click.pass_obj
def list_monitors(ctx, monitor_type, limit, namespace):
    monitor_types = None
    namespaces = [namespace] if namespace else None
    if monitor_type:
        if monitor_type == MonitorService.PSEUDO_MONITOR_TYPE_CB_COMPATIBLE:
            monitor_type = MonitorService.MONITOR_TYPE_CUSTOM_SQL
        monitor_types = [monitor_type]
    MonitorService().list_monitors(limit, namespaces, monitor_types)


@monitors.command(name="namespaces", help="List all namespaces.")
@click.option(
    "--limit",
    required=False,
    default=100,
    type=click.INT,
    show_default=True,
    help="Max number of namespaces to list.",
)
@click.pass_obj
def list_namespaces(ctx, limit):
    MonteCarloConfigService(config=ctx["config"]).list_namespaces(limit)



import json
import os
import pathlib
from unittest import TestCase
from unittest.mock import Mock, patch, call, ANY

import click
from box import Box
from pycarlo.core import Client

from montecarlodata.common.data import MonolithResponse
from montecarlodata.iac.mc_config_service import MonteCarloConfigService
from montecarlodata.queries.iac import CREATE_OR_UPDATE_MONTE_CARLO_CONFIG_TEMPLATE
from montecarlodata.utils import GqlWrapper
from tests.test_common_user import _SAMPLE_CONFIG


class MonteCarloConfigServiceTest(TestCase):
    @patch("os.getcwd")
    def setUp(self, getcwd) -> None:
        self._request_wrapper_mock = Mock(spec=GqlWrapper)
        self._pycarlo_client = Mock(spec=Client)
        self._print_func = Mock(spec=click.echo)
        self.project_dir = self._get_project_dir("standalone_configs")
        getcwd.return_value = self.project_dir
        self.service = MonteCarloConfigService(
            _SAMPLE_CONFIG,
            self._request_wrapper_mock,
            pycarlo_client=self._pycarlo_client,
            print_func=self._print_func,
        )

    def test_standalone_configs(self):
        files, template, _ = self.service.compile()
        self.assertEqual(len(files), 3)
        self.assertEqual(
            template,
            {
                "field_health": [
                    {"table": "analytics:prod.client_hub", "fields": ["account_id"]}
                ],
                "dimension_tracking": [
                    {"table": "analytics:prod.customer_360", "field": "account_id"}
                ],
            },
        )

    def test_embedded_dbt_configs(self):
        self.service = MonteCarloConfigService(
            _SAMPLE_CONFIG,
            self._request_wrapper_mock,
            project_dir=self._get_project_dir("embedded_dbt_configs"),
            pycarlo_client=self._pycarlo_client,
        )
        files, template, _ = self.service.compile()

        self.assertEqual(len(files), 4)
        self.assertEqual(
            template,
            {
                "field_health": [
                    {
                        "table": "analytics:prod_lineage.lineage_nodes",
                        "timestamp_field": "created",
                    },
                    {"table": "analytics:prod.abc"},
                    {"table": "analytics:prod.client_hub", "fields": ["account_id"]},
                ],
                "freshness": [
                    {
                        "table": "analytics:prod.abc",
                        "freshness_threshold": 30,
                        "schedule": {
                            "type": "fixed",
                            "interval_minutes": 30,
                            "start_time": "2021-07-27T19:51:00",
                        },
                    }
                ],
                "dimension_tracking": [
                    {"table": "analytics:prod.customer_360", "field": "account_id"}
                ],
            },
        )

    def test_invalid_configs(self):
        self.service = MonteCarloConfigService(
            _SAMPLE_CONFIG,
            self._request_wrapper_mock,
            project_dir=self._get_project_dir("invalid_configs"),
            pycarlo_client=self._pycarlo_client,
        )
        self.service._abort_on_error = False

        files, template, errors_by_file = self.service.compile(abort_on_error=False)
        errors = sorted(list(errors_by_file.items()), key=lambda x: x[0])

        file, error = errors[0]
        self.assertTrue(file.endswith("dir1/dir2/monitors.yml"))
        self.assertEqual(error, ['"custom_sql" property should be a list.'])
        file, error = errors[1]
        self.assertTrue(file.endswith("dir1/monitors.yml"))
        self.assertEqual(error, ['"field_health" property should be a list.'])

    def test_apply(self):
        namespace = "foo"
        self._request_wrapper_mock.make_request_v2.return_value = MonolithResponse(
            data={
                "response": {
                    "resourceModifications": [
                        {
                            "type": "ResourceModificationType.UPDATE",
                            "description": "Monitor: type=stats, table=analytics:prod.client_hub",
                            "resourceAsJson": '{"uuid": "ed4d07c3-58fd-44d0-8b2d-c1b020f45a69", "resource": null, "name": "monitor|type=stats|table=analytics:prod.client_hub|timestamp_field=<<NULL>>|where_condition=<<NULL>>", "table": "analytics:prod.customer_360", "type": "stats", "fields": [], "timestamp_field": null, "where_condition": null}',
                        },
                        {
                            "type": "ResourceModificationType.UPDATE",
                            "description": "Monitor: type=categories, table=analytics:prod.customer_360",
                            "resourceAsJson": '{"uuid": "ec3b0a80-d088-4dbe-acf5-150caf041574", "resource": null, "name": "monitor|type=categories|table=analytics:prod.customer_360|timestamp_field=<<NULL>>|where_condition=<<NULL>>|fields=account_id", "table": "analytics:prod.customer_360", "type": "categories", "fields": ["account_id"], "timestamp_field": null, "where_condition": null}',
                        },
                    ],
                    "changesApplied": True,
                    "errorsAsJson": "{}",
                    "warningsAsJson": "{}",
                }
            }
        )

        response = self.service.apply(namespace)

        config_template_as_dict = {
            "field_health": [
                {"table": "analytics:prod.client_hub", "fields": ["account_id"]}
            ],
            "dimension_tracking": [
                {"table": "analytics:prod.customer_360", "field": "account_id"}
            ],
        }

        self._request_wrapper_mock.make_request_v2.assert_called_once_with(
            query=CREATE_OR_UPDATE_MONTE_CARLO_CONFIG_TEMPLATE,
            operation="createOrUpdateMonteCarloConfigTemplate",
            variables=dict(
                namespace=namespace,
                configTemplateJson=json.dumps(config_template_as_dict),
                dryRun=False,
                misconfiguredAsWarning=True,
                resource=None,
            ),
        )

        self.assertEqual(response.errors, {})
        self.assertEqual(len(response.resource_modifications), 2)

    def test_apply_with_errors(self):
        namespace = "foo"
        self._request_wrapper_mock.make_request_v2.return_value = MonolithResponse(
            data={
                "response": {
                    "resourceModifications": [],
                    "changesApplied": False,
                    "errorsAsJson": '{"validation_errors": {"monitors": {"0": {"type": ["Unknown field."]}}}}',
                }
            }
        )

        response = self.service.apply(namespace, abort_on_error=False)

        config_template_as_dict = {
            "field_health": [
                {"table": "analytics:prod.client_hub", "fields": ["account_id"]}
            ],
            "dimension_tracking": [
                {"table": "analytics:prod.customer_360", "field": "account_id"}
            ],
        }

        self._request_wrapper_mock.make_request_v2.assert_called_once_with(
            query=CREATE_OR_UPDATE_MONTE_CARLO_CONFIG_TEMPLATE,
            operation="createOrUpdateMonteCarloConfigTemplate",
            variables=dict(
                namespace=namespace,
                configTemplateJson=json.dumps(config_template_as_dict),
                dryRun=False,
                misconfiguredAsWarning=True,
                resource=None,
            ),
        )

        self.assertEqual(
            response.errors,
            {"validation_errors": {"monitors": {"0": {"type": ["Unknown field."]}}}},
        )
        self.assertEqual(len(response.resource_modifications), 0)

    def test_misconfigured_warnings(self):
        namespace = "foo"
        self._request_wrapper_mock.make_request_v2.return_value = MonolithResponse(
            data={
                "response": {
                    "resourceModifications": [
                        {
                            "type": "ResourceModificationType.CREATE",
                            "description": "Freshness SLO: tables=['analytics:prod.client_hub', 'analytics:prod.client_warehouses'] freshness_threshold=1",
                            "resourceAsJson": '{"uuid": null, "resource": null, "name": "freshnessrule|tables=(multiple)", "description": null, "notes": null, "labels": [], "schedule": {"type": "fixed", "interval_minutes": 1, "interval_crontab": null, "start_time": "2021-07-27T19:00:00"}, "table": null, "tables": ["analytics:prod.client_hub", "analytics:prod.client_warehouses"], "freshness_threshold": 1}',
                        }
                    ],
                    "changesApplied": False,
                    "errorsAsJson": "{}",
                    "warningsAsJson": '{"misconfigured_warnings": [{"title": "High breaching Freshness SLOs are defined, adjust freshness_threshold in", "items": ["analytics:prod.client_hub: freshness_threshold is 1, the expected minimum threshold is 464", "analytics:prod.client_warehouses: freshness_threshold is 1, the expected minimum threshold is 1440"]}]}',
                }
            }
        )

        response = self.service.apply(namespace, abort_on_error=False)

        self._request_wrapper_mock.make_request_v2.assert_called_once_with(
            query=CREATE_OR_UPDATE_MONTE_CARLO_CONFIG_TEMPLATE,
            operation="createOrUpdateMonteCarloConfigTemplate",
            variables=dict(
                namespace=namespace,
                configTemplateJson=ANY,
                dryRun=False,
                misconfiguredAsWarning=True,
                resource=None,
            ),
        )

        self.assertEqual(
            response.warnings,
            {
                "misconfigured_warnings": [
                    {
                        "items": [
                            "analytics:prod.client_hub: freshness_threshold is 1, the expected "
                            "minimum threshold is 464",
                            "analytics:prod.client_warehouses: freshness_threshold is 1, the expected "
                            "minimum threshold is 1440",
                        ],
                        "title": "High breaching Freshness SLOs are defined, adjust freshness_threshold in",
                    }
                ]
            },
        )
        self.assertEqual(len(response.resource_modifications), 1)

    def _get_project_dir(self, dir_name: str):
        return os.path.join(
            pathlib.Path(__file__).parent.resolve(), "test_resources", dir_name
        )

    LIMIT = 2
    NAMESPACES_TABLE = """\
╒═════════════╤══════════════════════════════════╕
│ Namespace   │ Last Update Time                 │
╞═════════════╪══════════════════════════════════╡
│ namespace_1 │ 2000-01-01 00:00:00.000000+00:00 │
├─────────────┼──────────────────────────────────┤
│ namespace_2 │ 2000-01-01 00:00:00.000000+00:00 │
╘═════════════╧══════════════════════════════════╛"""

    @staticmethod
    def _mc_config_templates_response(namespace_count):
        return Box(
            {
                "get_monte_carlo_config_templates": {
                    "edges": [
                        {
                            "node": {
                                "namespace": f"namespace_{i}",
                                "last_update_time": "2000-01-01 00:00:00.000000+00:00",
                            }
                        }
                        for i in range(1, namespace_count + 1)
                    ]
                }
            }
        )

    def test_list_namespaces(self):
        self._pycarlo_client.return_value = self._mc_config_templates_response(
            self.LIMIT
        )
        self.service.list_namespaces(self.LIMIT)
        self._print_func.assert_called_once_with(self.NAMESPACES_TABLE)

    def test_list_namespaces_with_more_available(self):
        self._pycarlo_client.return_value = self._mc_config_templates_response(
            self.LIMIT + 1
        )
        self.service.list_namespaces(self.LIMIT)
        expected_calls = [
            call(self.NAMESPACES_TABLE),
            call(self.service.MORE_NS_MESSAGE),
        ]
        self._print_func.assert_has_calls(expected_calls)



from montecarlodata.config import Config
from montecarlodata.errors import manage_errors
from montecarlodata.integrations.onboarding.base import BaseOnboardingService
from montecarlodata.integrations.onboarding.fields import (
    SPARK_CONNECTION_TYPE,
    EXPECTED_SPARK_GQL_RESPONSE_FIELD,
)
from montecarlodata.queries.onboarding import (
    TEST_SPARK_BINARY_MODE_CRED_MUTATION,
    TEST_SPARK_HTTP_MODE_CRED_MUTATION,
    TEST_SPARK_DATABRICKS_CRED_MUTATION,
)

SPARK_BINARY_MODE_CONFIG_TYPE = "binary"
SPARK_HTTP_MODE_CONFIG_TYPE = "http"
SPARK_DATABRICKS_CONFIG_TYPE = "databricks"


class SparkOnboardingService(BaseOnboardingService):

    _MUTATIONS = {
        SPARK_BINARY_MODE_CONFIG_TYPE: TEST_SPARK_BINARY_MODE_CRED_MUTATION,
        SPARK_HTTP_MODE_CONFIG_TYPE: TEST_SPARK_HTTP_MODE_CRED_MUTATION,
        SPARK_DATABRICKS_CONFIG_TYPE: TEST_SPARK_DATABRICKS_CRED_MUTATION,
    }

    def __init__(self, config: Config, **kwargs):
        super().__init__(config, **kwargs)

    @manage_errors
    def onboard_spark(self, config_type: str, **kwargs) -> None:
        """
        Onboard a spark connection by validating and adding a connection
        """
        validation_query = self._MUTATIONS[config_type]
        kwargs["connectionType"] = SPARK_CONNECTION_TYPE
        self.onboard(
            validation_query=validation_query,
            validation_response=EXPECTED_SPARK_GQL_RESPONSE_FIELD,
            connection_type=SPARK_CONNECTION_TYPE,
            **kwargs
        )



import copy
from typing import Optional, Dict, List
from urllib import parse

import click
from tabulate import tabulate

from montecarlodata.collector.fields import (
    EXPECTED_GENERATE_TEMPLATE_GQL_RESPONSE_FIELD,
    DEFAULT_COLLECTION_REGION,
    EXPECTED_ADD_DC_RESPONSE_FIELD,
    ADD_DC_PROMPT_VERBIAGE,
    ADD_DC_REGION_PROMPT_VERBIAGE,
)
from montecarlodata.common.user import UserService
from montecarlodata.config import Config
from montecarlodata.errors import manage_errors, complain_and_abort, prompt_connection
from montecarlodata.queries.collector import (
    GENERATE_COLLECTOR_TEMPLATE,
    ADD_COLLECTOR_RECORD,
)
from montecarlodata.utils import GqlWrapper, AwsClientWrapper


class CollectorManagementService:
    _DEFAULT_COLLECTOR_VAL = "-"
    _LIST_COLLECTOR_HEADERS = [
        "AWS Stack ARN",
        "ID",
        "Version",
        "Template",
        "Last updated",
        "Active",
    ]
    _LEGACY_TEMPLATE = "cloudformation:July-2019"
    _JANUS_TEMPLATE_VARIANT = "janus"
    _JANUS_UNSUPPORTED_PARAMETERS = [
        "CreateCrossAccountRole",
        "EnableGatewayAccessLogging",
    ]
    _JANUS_TITLE_CASE_PARAMETERS = [
        "subnetCIDRs",
        "vpcCIDR",
        "workerLambdaConcurrentExecutions",
    ]

    def __init__(
        self,
        config: Config,
        request_wrapper: Optional[GqlWrapper] = None,
        aws_wrapper: Optional[AwsClientWrapper] = None,
        aws_profile_override: Optional[str] = None,
        aws_region_override: Optional[str] = None,
        user_service: Optional[UserService] = None,
    ):
        self._abort_on_error = True
        self._collector_region = (
            aws_region_override or config.aws_region or DEFAULT_COLLECTION_REGION
        )

        self._request_wrapper = request_wrapper or GqlWrapper(config)
        self._aws_wrapper = aws_wrapper or AwsClientWrapper(
            profile_name=aws_profile_override or config.aws_profile,
            region_name=self._collector_region,
        )
        self._user_service = user_service or UserService(
            request_wrapper=self._request_wrapper, config=config
        )

    @manage_errors
    def echo_template(self, **kwargs) -> None:
        """
        Echos the most recent template for this account
        """
        click.echo(
            self._get_template_url_from_launch(self._get_template_launch_url(**kwargs))
        )

    @manage_errors
    def echo_collectors(
        self,
        headers: Optional[str] = "firstrow",
        table_format: Optional[str] = "fancy_grid",
        active_only: Optional[bool] = False,
    ) -> None:
        """
        Echo collectors in a fancy grid.
        """
        table = [self._LIST_COLLECTOR_HEADERS]
        for collector in self._user_service.collectors or []:
            if active_only and not collector.get("active"):
                continue

            template_provider = collector.get("templateProvider")
            template_variant = collector.get("templateVariant")
            template_version = collector.get("templateVersion")

            if template_provider:
                template = f"{template_provider}:{template_variant}:{template_version}"
            else:
                template = self._LEGACY_TEMPLATE

            table.append(
                [
                    collector.get("stackArn")
                    if collector.get("stackArn")
                    else self._DEFAULT_COLLECTOR_VAL,
                    collector["uuid"],
                    collector.get("codeVersion")
                    or template_version
                    or self._DEFAULT_COLLECTOR_VAL,
                    template,
                    collector.get("lastUpdated")
                    if collector.get("lastUpdated")
                    else self._DEFAULT_COLLECTOR_VAL,
                    collector.get("active"),
                ]
            )
        click.echo(tabulate(table, headers=headers, tablefmt=table_format))

    @manage_errors
    def add_collector(self, no_prompt: Optional[bool] = False) -> None:
        """
        Add a collector to the account. Prompts to launch browser.
        """
        response = self._request_wrapper.make_request_v2(
            query=ADD_COLLECTOR_RECORD,
            operation=EXPECTED_ADD_DC_RESPONSE_FIELD,
        )
        dc_id = response.data.dc.uuid
        click.echo(f"Created collector record with ID '{dc_id}'")
        if no_prompt:
            return

        prompt_connection(message=ADD_DC_PROMPT_VERBIAGE)  # exits on No
        collection_region = click.prompt(
            ADD_DC_REGION_PROMPT_VERBIAGE,
            show_choices=True,
            default=DEFAULT_COLLECTION_REGION,
            show_default=True,
            type=click.Choice(self._user_service.active_collection_regions),
        )
        self.launch_quick_create_link(
            dry=False, dc_id=dc_id, collection_region=collection_region
        )

    @manage_errors
    def launch_quick_create_link(self, dry: Optional[bool] = True, **kwargs) -> None:
        """
        Open browser with a quick create link for deploying a data collector

        https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/cfn-console-create-stacks-quick-create-links.html
        """
        launch_url = self._get_template_launch_url(**kwargs)
        if dry:
            click.echo(launch_url)
            return
        click.launch(launch_url)

    @manage_errors
    def upgrade_template(
        self, update_infra: bool = False, new_params: Optional[Dict] = None, **kwargs
    ) -> None:
        """
        Upgrades the DC attached to this account
        """
        collector_details = self._generate_collector_template(
            update_infra=update_infra, **kwargs
        )
        try:
            stack_arn = collector_details["stackArn"]
            is_active = collector_details["active"]
            gateway_id = collector_details["apiGatewayId"]
            template_variant = collector_details["templateVariant"]
            template_url = self._get_template_url_from_launch(
                collector_details["templateLaunchUrl"]
            )

            if not is_active:
                complain_and_abort(
                    "Cannot upgrade an inactive collector. Please contact Monte Carlo"
                )
        except KeyError as err:
            complain_and_abort(
                f"Missing expected property ({err}). The collector may not have been deployed before"
            )
        else:
            click.echo(f"Updating '{stack_arn}'")
            update_variant = (
                update_infra and template_variant != self._JANUS_TEMPLATE_VARIANT
            )
            self._upgrade(
                stack_arn=stack_arn,
                template_url=template_url,
                gateway_id=gateway_id,
                new_params=new_params,
                update_variant=update_variant,
            )
            click.echo("Upgrade complete! Have a nice day!")

    @manage_errors
    def deploy_template(
        self,
        stack_name: str,
        termination_protection: bool,
        new_params: Optional[Dict] = None,
        **kwargs,
    ) -> None:
        """
        Deploys a template for this account
        """
        click.echo(f"Deploying '{stack_name}' in '{self._collector_region}'")
        collector_details = self._generate_collector_template(**kwargs)
        is_active = collector_details["active"]
        if is_active:
            complain_and_abort("Cannot deploy with an already active collector.")

        template_url = self._get_template_url_from_launch(
            collector_details["templateLaunchUrl"]
        )
        self._deploy(
            stack_name=stack_name,
            template_url=template_url,
            termination_protection=termination_protection,
            new_params=new_params,
        )

    def _upgrade(
        self,
        stack_arn: str,
        template_url: str,
        gateway_id: str,
        new_params: Dict,
        update_variant: bool,
    ) -> None:
        """
        Upgrade the collector stack and deploy the gateway
        """
        parameters = self._update_parameters(
            stack_arn=stack_arn,
            param_overrides=new_params,
            update_variant=update_variant,
        )
        if self._aws_wrapper.upgrade_stack(
            stack_id=stack_arn, template_link=template_url, parameters=parameters
        ):
            self._aws_wrapper.deploy_gateway(gateway_id=gateway_id)
        else:
            complain_and_abort("Failed to upgrade. Please review CF events for details")

    def _deploy(
        self,
        stack_name: str,
        template_url: str,
        termination_protection: bool,
        new_params: Optional[Dict] = None,
    ) -> None:
        """
        Deploy (create) a collector stack for the first time
        """
        parameters = self._build_param_list(existing_params=[], new_params=new_params)
        if not self._aws_wrapper.create_stack(
            stack_name=stack_name,
            template_link=template_url,
            termination_protection=termination_protection,
            parameters=parameters,
        ):
            complain_and_abort("Failed to deploy. Please review CF events for details")

    def _generate_collector_template(
        self, update_infra: bool = False, **kwargs
    ) -> Optional[Dict]:
        """
        Generate the latest template and returns any associated dc properties (i.e. from the DataCollectorModel)
        """
        options = {"region": self._collector_region, "update_infra": update_infra}
        if kwargs.get("collection_region"):
            options["region"] = kwargs["collection_region"]  # overwrite client

        if kwargs.get("dc_id"):
            options["dc_id"] = kwargs["dc_id"]

        response = self._request_wrapper.make_request_v2(
            query=GENERATE_COLLECTOR_TEMPLATE,
            operation=EXPECTED_GENERATE_TEMPLATE_GQL_RESPONSE_FIELD,
            variables=options,
        )
        return response.data.get("dc")

    def _get_template_launch_url(self, **kwargs) -> str:
        """
        Get the template launch url
        """
        return self._generate_collector_template(**kwargs).get("templateLaunchUrl")

    @staticmethod
    def _get_template_url_from_launch(launch_url: str) -> str:
        """
        Extract the template url from expected launch url structure
        """
        return parse.parse_qs(parse.urlparse(launch_url).fragment).get(
            "templateURL", []
        )[0]

    def _update_parameters(
        self, stack_arn: str, param_overrides: Optional[Dict], update_variant: bool
    ):
        existing_params = self._aws_wrapper.get_stack_parameters(stack_id=stack_arn)
        # we only allow updates to janus template variant
        if update_variant:
            converted_parameters, removed_parameters = self._adapt_parameters_for_janus(
                existing_params
            )

            combined_parameters = (
                self._build_param_list(
                    existing_params=existing_params,
                    # add the converted parameters, and consider overrides
                    new_params={**converted_parameters, **(param_overrides or {})},
                )
                or []
            )

            # remove any parameters that are no longer supported by janus template variant
            new_parameters = [
                param
                for param in combined_parameters
                if param["ParameterKey"] not in removed_parameters
            ]
            return new_parameters

        return self._build_param_list(existing_params, param_overrides)

    @staticmethod
    def _adapt_parameters_for_janus(parameters: Dict):
        converted_parameters = {}
        removed_parameters = []
        for parameter in parameters or []:
            key = parameter["ParameterKey"]
            if key in CollectorManagementService._JANUS_TITLE_CASE_PARAMETERS:
                title_case_key = key[0].upper() + key[1:]
                converted_parameters[title_case_key] = parameter["ParameterValue"]
                removed_parameters.append(key)
            elif key in CollectorManagementService._JANUS_UNSUPPORTED_PARAMETERS:
                removed_parameters.append(key)

        return converted_parameters, removed_parameters

    @staticmethod
    def _build_param_list(
        existing_params: List[Dict], new_params: Optional[Dict] = None
    ) -> List[Dict]:
        """
        Get a list of parameters by replacing (or extending) any new params into the existing stack params
        """
        existing_params = copy.deepcopy(existing_params)
        new_params = copy.deepcopy(new_params)

        for param in existing_params or []:
            if new_params and new_params.get(param["ParameterKey"]) is not None:
                param["ParameterValue"] = new_params[param["ParameterKey"]]
                param["UsePreviousValue"] = False
                del new_params[param["ParameterKey"]]
            else:
                del param["ParameterValue"]
                param["UsePreviousValue"] = True
        if new_params:
            # Handle any completely new params (i.e. those that were not in the existing struct)
            # Can be found for params added between versions
            existing_params.extend(
                [
                    {"ParameterKey": k, "ParameterValue": v}
                    for k, v in new_params.items()
                ]
            )

        return existing_params
